using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;

namespace Editroid
{
	internal partial class ComboEditor:UserControl
	{
		Graphic.Blitter b = new Editroid.Graphic.Blitter();
        bool isDrawing;

		Bitmap tiles = new Bitmap(256, 256, PixelFormat.Format8bppIndexed);

		public ComboEditor() {
			//InitializeComponent();


            this.lblSelection = new System.Windows.Forms.Label();
            this.SuspendLayout();
            // 
            // lblSelection
            // 
            this.lblSelection.BackColor = System.Drawing.Color.FromArgb(((int)(((byte)(96)))), ((int)(((byte)(255)))), ((int)(((byte)(255)))), ((int)(((byte)(255)))));
            this.lblSelection.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.lblSelection.Location = new System.Drawing.Point(0, 0);
            this.lblSelection.Name = "lblSelection";
            this.lblSelection.Size = new System.Drawing.Size(16, 16);
            this.lblSelection.TabIndex = 0;
            this.lblSelection.MouseMove += new System.Windows.Forms.MouseEventHandler(this.lblSelection_MouseMove);
            this.lblSelection.MouseDown += new System.Windows.Forms.MouseEventHandler(this.lblSelection_MouseDown);
            this.lblSelection.MouseUp += new System.Windows.Forms.MouseEventHandler(this.lblSelection_MouseUp);
            // 
            // ComboEditor
            // 
            this.BackColor = System.Drawing.Color.Black;
            this.Controls.Add(this.lblSelection);
            this.Name = "ComboEditor";
            this.ResumeLayout(false);

		}

		//public void SetPalette(NesPalette pal, int paltable) {
		//    ColorPalette p = tiles.Palette;

		//    pal.ApplyTable(p.Entries, 0, paltable * 4, 4);
		//}
		public void SetPalette(NesPalette pal) {
			ColorPalette p = tiles.Palette;
			pal.ApplyTable(p.Entries);
			tiles.Palette = p;

			this.DoubleBuffered = true;
		}

		private int _PaletteTable;

		public int PaletteTable {
			get { return _PaletteTable; }
			set { _PaletteTable = value; }
		}
	

		ComboTable _Combos;
		PatternTable patterns;
		public void SetCombos(ComboTable combos) {
			_Combos = combos;
		}

		public void SetPatterns(PatternTable p) {
			patterns = p;
		}

		private int _ScrollPos;

		public int ScrollPosition {
			get { return _ScrollPos; }
			set { _ScrollPos = value; }
		}


		/// <summary>
		/// Draws the combos in the editor.
		/// </summary>
		public void Draw() {
			b.Begin(patterns.Patterns, tiles);

			for(int row = 0; row < 16; row++) {
				for(int combo = 0; combo < 16; combo++ ) {
					int comboIndex = combo + (row + _ScrollPos)* 16;
					b.BlitTile(_Combos[comboIndex].GetByte(0), combo * 2, row * 2, (byte)_PaletteTable);
					b.BlitTile(_Combos[comboIndex].GetByte(1), combo * 2 + 1, row * 2, (byte)_PaletteTable);
					b.BlitTile(_Combos[comboIndex].GetByte(2), combo * 2, row * 2 + 1, (byte)_PaletteTable);
					b.BlitTile(_Combos[comboIndex].GetByte(3), combo * 2 + 1, row * 2 + 1, (byte)_PaletteTable);
				}
			}

			b.End();
			this.BackgroundImage = tiles;

			Invalidate();
		}

		private byte _CurrentTile;

		/// <summary>
		/// Gets/sets the tile that will be painted when the user clicks on a combo.
		/// </summary>
		public byte CurrentTile {
			get { return _CurrentTile; }
			set { _CurrentTile = value; }
		}


		/// <summary>
		/// Overrides base member.
		/// </summary>
		/// <param name="e">e argument.</param>
		protected override void OnMouseDown(MouseEventArgs e) {
			base.OnMouseDown(e);

			ProcessMouse(e);

            if(ModifierKeys == Keys.None && e.Button != MouseButtons.Right) { // Make selection
                int selX = e.X / 16;
                int selY = e.Y / 16;
                if(selX < 0) selX = 0; if(selX > 15) selX = 15;
                if(selY < 0) selY = 0; if(selY > 15) selY = 15;

                lblSelection.Location = new Point(selX * 16, selY * 16);
                OnSelectionChanged();
            }
		}

		/// <summary>
		/// Overrides base member.
		/// </summary>
		/// <param name="e">e argument.</param>
		protected override void OnMouseMove(MouseEventArgs e) {
			base.OnMouseMove(e);

			ProcessMouse(e);
		}
        protected override void OnMouseUp(MouseEventArgs e) {
            base.OnMouseUp(e);

            if(e.Button == MouseButtons.Left && isDrawing) {
                isDrawing = false;
                OnComboEdited();
            }
        }

        /// <summary>
        /// Processes mouse events for combo editing.
        /// </summary>
        /// <param name="e">Mouse event args for a mouse event.</param>
		private void ProcessMouse(MouseEventArgs e) {
			if(e.Button == MouseButtons.Left) {
				if(ModifierKeys == Keys.Control) { // Copy entire combo from tiles
					int Combo = e.X / 16 + (e.Y / 16) * 16;
					Combo ClickedCombo = _Combos[Combo];
					ClickedCombo[0] = (byte)_CurrentTile;
					if(_CurrentTile < 254)
						ClickedCombo[1] = (byte)(_CurrentTile + 1);
					if(_CurrentTile < 253)
						ClickedCombo[2] = (byte)(_CurrentTile + 2);
					if(_CurrentTile < 252)
						ClickedCombo[3] = (byte)(_CurrentTile + 3);

                    // Show changes
                    DrawCombo(e.X / 16, e.Y / 16);
                    this.Invalidate(new Rectangle((e.X / 16) * 16,
                                                (e.Y / 16) * 16,
                                                16, 16));
                    isDrawing = true;
                } else if(ModifierKeys == Keys.Shift) { // Copy single tile
                    int tileX = e.X / 8;
                    int tileY = (e.Y + _ScrollPos * 16) / 8;
                    int Combo = tileX / 2 + (tileY / 2) * 16;
                    int ComboTileX = tileX % 2;
                    int ComboTileY = tileY % 2;

                    Combo ClickedCombo = _Combos[Combo];
                    ClickedCombo[ComboTileX + ComboTileY * 2] = _CurrentTile;

                    // Show changes
                    DrawCombo(e.X / 16, e.Y / 16);
                    this.Invalidate(new Rectangle((e.X / 16) * 16,
                                                (e.Y / 16) * 16,
                                                16, 16));
                    isDrawing = true;
                }

            } else if(e.Button == MouseButtons.Right) {
                // On right click, select the clicked combo for painting
                int tileX = e.X / 8;
                int tileY = (e.Y + _ScrollPos * 16) / 8;
                int Combo = tileX / 2 + (tileY / 2) * 16;
                int ComboTileX = tileX % 2;
                int ComboTileY = tileY % 2;

                Combo ClickedCombo = _Combos[Combo];
                _CurrentTile = ClickedCombo[ComboTileX + ComboTileY * 2];
                if(UserGrabbedCombo != null)
                    UserGrabbedCombo(this, null);
            }
		}

        public event EventHandler SelectionChanged;
        public event EventHandler ComboEdited;
        public event EventHandler UserGrabbedCombo;
        protected virtual void OnSelectionChanged() { if(SelectionChanged != null) SelectionChanged(this, new EventArgs()); }
        protected virtual void OnComboEdited() { if(ComboEdited != null) ComboEdited(this, new EventArgs()); }


        public int SelectedCombo { get { return lblSelection.Left / 16 + 16 * (lblSelection.Top / 16); } }
		/// <summary>
		/// Draws a combo in the editor.
		/// </summary>
		/// <remarks>This function does not refresh the control.</remarks>
		public void DrawCombo(int X, int Y) {
			b.Begin(patterns.Patterns, tiles);

			int comboIndex = X + (Y + _ScrollPos) * 16;
			b.BlitTile(_Combos[comboIndex].GetByte(0), X * 2, Y * 2, (byte)_PaletteTable);
			b.BlitTile(_Combos[comboIndex].GetByte(1), X * 2 + 1, Y * 2, (byte)_PaletteTable);
			b.BlitTile(_Combos[comboIndex].GetByte(2), X * 2, Y * 2 + 1, (byte)_PaletteTable);
			b.BlitTile(_Combos[comboIndex].GetByte(3), X * 2 + 1, Y * 2 + 1, (byte)_PaletteTable);

			b.End();
		}

        private void lblSelection_MouseDown(object sender, MouseEventArgs e) {
            OnMouseDown(new MouseEventArgs(e.Button, e.Clicks, e.X + lblSelection.Left, e.Y + lblSelection.Top, e.Delta));
        }

        private void lblSelection_MouseMove(object sender, MouseEventArgs e) {
            OnMouseMove(new MouseEventArgs(e.Button, e.Clicks, e.X + lblSelection.Left, e.Y + lblSelection.Top, e.Delta));

        }

        private void lblSelection_MouseUp(object sender, MouseEventArgs e) {
            OnMouseUp(new MouseEventArgs(e.Button, e.Clicks, e.X + lblSelection.Left, e.Y + lblSelection.Top, e.Delta));
        }
	}
}
