using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Editroid
{
    internal class GlobalEventManager
    {
        public event EventHandler<LevelEventArgs> PaletteChanged;
        public event EventHandler<LevelEventArgs> StructureChanged;
        public event EventHandler<ComboEventArgs> ComboChanged;
        public event EventHandler<ScreenObjectEventArgs> ObjectSelected;
        public event EventHandler<ScreenObjectEventArgs> ObjectEdited;

        public void OnPaletteChanged(Levels level) {
            if(PaletteChanged != null)
                PaletteChanged(this, new LevelEventArgs(level));
        }
        public void OnStructureChanged(Levels level) {
            if(StructureChanged != null)
                StructureChanged(this, new LevelEventArgs(level));
        }
        public void OnComboChanged(Levels level, int ComboIndex) {
            if(ComboChanged != null)
                ComboChanged(this, new ComboEventArgs(level, ComboIndex));
        }
        public void OnObjectSelected(Control sender, Levels level, ScreenItem item) {
            if(ObjectSelected != null)
                ObjectSelected(sender, new ScreenObjectEventArgs(level, item));
        }
        public void OnObjectEdited(Control sender, Levels level, ScreenItem item) {
            if(ObjectEdited != null)
                ObjectEdited(sender, new ScreenObjectEventArgs(level, item));
        }

        private GlobalEventManager(){}
        static GlobalEventManager manager = new GlobalEventManager();
        public static GlobalEventManager Manager { get { return manager; } }

    }

    /// <summary>Provides information for an event pertaining to a level.</summary>
    public class LevelEventArgs: EventArgs
    {
        Levels level;
        /// <summary>The level for which the event occurred.</summary>
        public Levels Level { get { return level; } }
        /// <summary>Instantiates this class.</summary>
        public LevelEventArgs(Levels level) {
            this.level = level;
        }
    }

    /// <summary>Provides information for an event pertaining to a combo.</summary>
    public class ComboEventArgs:LevelEventArgs
    {
        int comboIndex;
        /// <summary>The index of the combo for which this event occurred.</summary>
        public int ComboIndex { get { return ComboIndex; } }
        /// <summary>Instantiates this class.</summary>
        public ComboEventArgs(Levels level, int comboIndex)
            : base(level) {
            this.comboIndex = comboIndex;
        }
    }

    /// <summary>Provides information for an event pertaining to a screen object.</summary>
    public class ScreenObjectEventArgs:LevelEventArgs
    {
        ScreenItem item;
        /// <summary>The item this event pertains to.</summary>
        public ScreenItem Item { get { return item; } }
        /// <summary></summary>
        /// <param name="level"></param>
        /// <param name="item"></param>
        public ScreenObjectEventArgs(Levels level, ScreenItem item)
            : base(level) {
            this.item = item;
        }
    }
}
