using System;
using System.Collections.Generic;
using System.Text;
using Editroid.ROM;

namespace Editroid.Graphic
{
    /// <summary>
    /// Sprites used for power-ups.
    /// </summary>
    public static class ItemSprites
    {
        /// <summary>Sprite for elevators.</summary>
        public static Sprite Elevator = new Sprite(new byte[]{SpriteMacros.MoveR4, 0x28, 0x38, SpriteMacros.FlipX, 0x28});
        /// <summary>Sprite for a palette switch.</summary>
        public static Sprite PalSwitch = new Sprite(new byte[] { 0x6E, 0x6F, SpriteMacros.NextRow, 0x6F, 0x6E });
    }

    /// <summary>
    /// Defines sprites for power-ups.
    /// </summary>
    public static class PowerUpSprites
    {
        /// <summary>Sprite for the bombs.</summary>
        public static Sprite Bombs = new Sprite(new byte[] { 0x86, 0x87, SpriteMacros.NextRow, 0x96, 0x97 });
        /// <summary>Sprite for high jump boots.</summary>
        public static Sprite HighJump = new Sprite(new byte[] { 0x7B, 0x7C, SpriteMacros.NextRow, 0x8B, 0x8C });
        /// <summary>Sprite for the longbeam.</summary>
        public static Sprite LongBeam = new Sprite(new byte[] { 0x88, 0x67, SpriteMacros.NextRow, 0x98, 0x99 });
        /// <summary>Sprite for the screw attack.</summary>
        public static Sprite ScrewAttack = new Sprite(new byte[] { 0x80, 0x81, SpriteMacros.NextRow, 0x90, 0x91 });
        /// <summary>Sprite for the maru mari.</summary>
        public static Sprite MaruMari = new Sprite(new byte[] { 0x7D, 0x7E, SpriteMacros.NextRow, 0x8D, 0x8E });
        /// <summary>Sprite for the varia suit.</summary>
        public static Sprite Varia = new Sprite(new byte[] { 0x82, 0x83, SpriteMacros.NextRow, 0x92, 0x93 });
        /// <summary>Sprite for the wavebeam.</summary>
        public static Sprite WaveBeam = new Sprite(new byte[] { 0x88, 0x8A, SpriteMacros.NextRow, 0x98, 0x99 });
        /// <summary>Sprite for the icebeam.</summary>
        public static Sprite IceBeam = new Sprite(new byte[] { 0x88, 0x68, SpriteMacros.NextRow, 0x98, 0x99 });
        /// <summary>Sprite for an energy tank.</summary>
        public static Sprite EnergyTank = new Sprite(new byte[] { 0x84, 0x85, SpriteMacros.NextRow, 0x94, 0x95 });
        /// <summary>Sprite for missiles.</summary>
        public static Sprite Missile = new Sprite(new byte[] { 0x3F, SpriteMacros.FlipX, 0x3F, SpriteMacros.NextRow, 0x4F, SpriteMacros.FlipX, 0x4F });
        /// <summary>Sprite for dot missiles.</summary>
        public static Sprite MissileDot = new Sprite(new byte[] { 0x3F, SpriteMacros.FlipX, 0x3F, SpriteMacros.NextRow, 0x4F, SpriteMacros.FlipX, 0x4F });
        /// <summary>Sprite for arrow missiles.</summary>
        public static Sprite MissileArrow = new Sprite(new byte[] { 0x3F, SpriteMacros.FlipX, 0x3F, SpriteMacros.NextRow, 0x4F, SpriteMacros.FlipX, 0x4F });
        /// <summary>Sprite for unknow object.</summary>
        public static Sprite UnknownC = new Sprite(new byte[] {});
        /// <summary>Sprite for unknow object.</summary>
        public static Sprite UnknownD = new Sprite(new byte[] { });
        /// <summary>Sprite for unknow object.</summary>
        public static Sprite UnknownE = new Sprite(new byte[] { });
        /// <summary>Sprite for unknow object.</summary>
        public static Sprite UnknownF = new Sprite(new byte[] { });

        /// <summary>
        /// Gets the sprite for the specified power-up.
        /// </summary>
        /// <param name="powerup">The type of power-up to get a sprite for.</param>
        /// <returns>A Sprite object.</returns>
        public static Sprite GetSprite(PowerUpType powerup){
            switch(powerup) {
                case PowerUpType.Bomb:
                    return Bombs;
                case PowerUpType.HiJump:
                    return HighJump;
                case PowerUpType.LongBeam:
                    return LongBeam;
                case PowerUpType.ScrewAttack:
                    return ScrewAttack;
                case PowerUpType.MaruMari:
                    return MaruMari;
                case PowerUpType.Varia:
                    return Varia;
                case PowerUpType.WaveBeam:
                    return WaveBeam;
                case PowerUpType.IceBeam:
                    return IceBeam;
                case PowerUpType.EnergyTank:
                    return EnergyTank;
                case PowerUpType.Missile:
                    return Missile;
                case PowerUpType.Invalid_Missile_Dot:
                    return MissileDot;
                case PowerUpType.Invalid_Missile_Arrow:
                    return MissileArrow;
                case PowerUpType.Unknown_c:
                    return UnknownC;
                case PowerUpType.Unknown_d:
                    return UnknownD;
                case PowerUpType.Unknown_e:
                    return UnknownE;
                default:
                    return UnknownF;
            }
        }
    }
}
