using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Editroid.ROM;
using System.Drawing;

namespace Editroid
{
    class ItemEntryEditor: Panel
    {
        ItemEntry entry;

        public ItemEntryEditor(ItemEntry e) {
            entry = e;

            // Initialize control
            BackColor = Color.FromArgb(0x60, Color.White);
            Height = 8;

            // Populate control
            ItemSeeker seeker = e.Seek();
            do {
                ItemDatEditor datEditor = new ItemDatEditor(seeker, e);
                Controls.Add(datEditor);
            } while(seeker.NextScreen());

            // Position Control
            Top = e.MapY * 8;
            Trim();
        }

        public void Trim() {
            int targetLeft = ((Controls[0] as ItemDatEditor).MapX) * 8;
            bool resetLefts = targetLeft != Left;
            int targetWidth = 8;

            if(resetLefts)
                this.Left = targetLeft;

            foreach(Control c in Controls) {
                if(resetLefts)
                    ((ItemDatEditor)c).ResetLeft();
                if(c.Right > targetWidth)
                    targetWidth = c.Right;
            }

            if(targetWidth != this.Width)
                this.Width = targetWidth;


            //int width = 8;
            //foreach(Control c in Controls) {
            //    ((ItemDatEditor)c).ResetLeft();
            //    if(c.Right > width)
            //        width = c.Right;
            //}

            //Width = width;
        }


        public ItemEntry Entry { get { return entry; } }
        public int MapY {
            get {
                return entry.MapY;
            }
            set{
                entry.MapY = value;
                Top = value * 8;
            }
        }

        /// <summary>
        /// Ensures that the target position of an ItemDatEditor is valid.
        /// </summary>
        /// <param name="item">The itemDatEditor to check the target position of.</param>
        /// <param name="destX">The target to be checked, and, if needed, modified.</param>
        internal void VerifyItemPosition(ItemDatEditor item, ref int destX) {
            foreach(ItemDatEditor editor in Controls) {
                int position = item.ComparePosition(editor);
                // If an item A is to the left of item B and the user tries to move B to the left of A,
                if(position < 0 && destX <= editor.MapX)
                    // Move B one spot right of A.
                    destX = editor.MapX + 1;
                // If an item A is to the right of item B and the user tries to move B to the right of A,
                if(position > 0 && destX >= editor.MapX)
                    // Move B one spot left of A.
                    destX = editor.MapX - 1;
            }
        }

        internal void RequestVerticalMove(int p) {
            int destY = this.Top / 8 + p;
            if(destY < 1) destY = 1;
            if(destY > 30) destY = 30;

            //bool checkCollision = true;
            //while(checkCollision){
            //    checkCollision = false;

                foreach(ItemEntryEditor e in Parent.Controls) {
                    if(e != this) {
                        if(e.MapY == destY) {
                            return;
                            //destY++;
                            //if(destY == 30) destY = 1;
                            //checkCollision = true;
                        }
                    }
                }
            //}

            this.MapY = destY;
        }
    }

    class ItemDatEditor:Label
    {
        ItemSeeker item;
        ItemEntry entry;

        public ItemDatEditor(ItemSeeker item, ItemEntry entry) {
            this.item = item;
            this.entry = entry;

            // Initialize
            AutoSize = false;
            Left = item.MapX * 8;
            Size = new System.Drawing.Size(8, 8);
            Text = null;
            Image = ItemEditTool.GetEditTool(item).GetImage(item);
        }

        /// <summary>
        /// Compares the position of items.
        /// </summary>
        /// <param name="d">The item to compare to.</param>
        /// <returns>A negative value if the compared item comes before this item, a positive value if it comes after, or zero if they are at the same position.</returns>
        public int ComparePosition(ItemDatEditor d) {
            if(d.Item.MapX < MapX) return -1;
            if(d.Item.MapX > MapX) return 1;
            return 0;
        }

        public ItemEntryEditor EntryEditor { get { return this.Parent as ItemEntryEditor; } }
        public ItemSeeker Item { get { return item; } }
        public int MapX {
            get {
                return item.MapX;
            }
            set {
                item.MapX = value;
                ResetLeft();
            }
        }

        /// <summary>
        /// Resets the position of this control based on the parent's position.
        /// </summary>
        public void ResetLeft() {
            Left = item.MapX * 8 - Parent.Left;
            EntryEditor.Trim();
        }

        bool dragging;
        int dragX, dragY;
        protected override void OnMouseDown(MouseEventArgs e) {
            if(e.Button == MouseButtons.Left) {
                dragging = true;
                dragX = e.X / 8;
                dragY = e.Y / 8;
            }

            base.OnMouseDown(e);

            (this.FindForm() as ItemEditor).SelectItem(this);
        }

        protected override void OnMouseMove(MouseEventArgs e) {
            if(dragging) {
                int destX = MapX + e.X / 8;
                if(e.X < 0) destX--; // Negative division rounds up instead of down
                if(destX < 1) destX = 1; if(destX > 30) destX = 30;
                int destY = e.Y / 8;
                if(e.Y < 0) destY--; // Negative division rounds up instead of down

                bool dragHorizontal = destX != MapX;
                bool dragVertical = destY != 0;

                if(dragHorizontal) {
                    EntryEditor.VerifyItemPosition(this, ref destX);
                    MapX = destX;
                }
                if(dragVertical) {
                    EntryEditor.RequestVerticalMove(destY);
                }
            }

            base.OnMouseMove(e);
        }
        protected override void OnMouseUp(MouseEventArgs e) {
            if(e.Button == MouseButtons.Left)
                dragging = false;

            base.OnMouseUp(e);
        }

        bool Leftmost {
            get {
                return this == Parent.Controls[0];
            }
        }

    }
}
