using System;
using System.Collections.Generic;
using System.Text;
using Editroid.Graphic;
using System.Drawing;

namespace Editroid.ROM
{
    /// <summary>
    /// Represents a power-up.
    /// </summary>
    /// <remarks>This class is not used by Editroid.</remarks>
    public class GameItem
    {
        byte[] data;
        int offset;
        Sprite sprite;

        int screenX = -1;
        int screenY = -1;
        int pal = 0;

        /// <summary>
        /// Creates a GameItem.
        /// </summary>
        /// <param name="data">The ROM data to load this from.</param>
        /// <param name="offset">The location in ROM data the item data is found at.</param>
        /// <param name="graphic">The sprite that represents this item.</param>
        /// <param name="palette">The palette this item is rendered with.</param>
        public GameItem(byte[] data, int offset, Sprite graphic, int palette) {
            this.data = data;
            this.offset = offset;
            this.sprite = graphic;
            this.pal = palette;
        }

        /// <summary>
        /// Creates a GameItem.
        /// </summary>
        /// <param name="data">The ROM data to load this from.</param>
        /// <param name="offset">The location in ROM data the item data is found at.</param>
        /// <param name="graphic">The sprite that represents this item.</param>
        /// <param name="palette">The palette this item is rendered with.</param>
        /// <param name="screenLocation">The location within the screen this item is found at.</param>
        public GameItem(byte[] data, int offset, Sprite graphic, int palette, Point screenLocation) {
            this.data = data;
            this.offset = offset;
            this.sprite = graphic;

            this.screenX = screenLocation.X;
            this.screenY = screenLocation.Y;
            this.pal = palette;
        }


        /// <summary>
        /// The location, on the map, where this item is found.
        /// </summary>
        public Point MapLocation {
            get {
                return new Point(data[offset + 3], data[offset]);
            }
        }

        /// <summary>
        /// The location, within the screen, where this item is found.
        /// </summary>
        public Point ScreenLocation {
            get {
                Point result = new Point(screenX, screenY);
                if(screenY == -1)
                    result.Y = 2 * data[offset + 7] / 0x10;
                if(screenX == -1)
                    result.X = 2 * data[offset + 7] & 0x0F;

                return result;
            }
        }

        /// <summary>
        /// Draws this item.
        /// </summary>
        /// <param name="b"></param>
        public void Draw(Blitter b) {
            // Todo: add palette property and pram in .ctor
            sprite.Draw(b, ScreenLocation.X, ScreenLocation.Y, (byte)pal); 
        }

        /// <summary>
        /// Gets a string representation of this object.
        /// </summary>
        /// <returns>A string representation of this object.</returns>
        public override string ToString() {
            return "GameItem: " +
                "MapLocation=" + MapLocation.X.ToString() + "," + MapLocation.Y.ToString() +
                " ScreenLocation=" + ScreenLocation.X.ToString() + "," + ScreenLocation.Y.ToString() +
                " @" + offset.ToString("x");
        }
    }

}
