using System;
using System.Collections.Generic;
using System.Text;

namespace Editroid
{
	/// <summary>
	/// Contains the offsets of pointer tables for the five different levels
	/// </summary>
	public class LevelPointers
	{
		/// <summary>
		/// Location of this levels room data pointer table
		/// </summary>
		internal readonly PointerTable rooms;
		/// <summary>
		/// Location of this levels struct data pointer table
		/// </summary>
        internal readonly PointerTable structs;

        /// <summary>
        /// Gets the pointer table for rooms in this level.
        /// </summary>
        public PointerTable Rooms { get { return rooms; } }
        /// <summary>
        /// Gets the pointer table for structs in this level.
        /// </summary>
        public PointerTable Structs { get { return structs; } }

		internal LevelPointers(int roomoffset, int roomlen, int structoffset, int structlen) {
			rooms.Offset = roomoffset;
			rooms.Length = roomlen;
			structs.Offset = structoffset;
			structs.Length = structlen;
		}

		/// <summary>
		/// Level pointers for Brinstar
		/// </summary>
		public readonly static LevelPointers Brinstar = new LevelPointers(0x6324, 0x2F, 0x6382, 0x32);
		/// <summary>
		/// Level pointers for Norfair
		/// </summary>
		public readonly static LevelPointers Norfair = new LevelPointers(0xA22B, 0x2E, 0xA287, 0x31);
		/// <summary>
		/// Level pointers for Tourain
		/// </summary>
		public readonly static LevelPointers Tourain = new LevelPointers(0xE7E1, 0x15, 0xE80B, 0x20);
		/// <summary>
		/// Level pointers for Kraid's Hideout
		/// </summary>
		public readonly static LevelPointers Kraid = new LevelPointers(0x121E5, 0x25, 0x1222F, 0x27);
		/// <summary>
		/// Level pointers for Ridley's Hideout
		/// </summary>
		public readonly static LevelPointers Ridley = new LevelPointers(0x1618F, 0x2A, 0x161E3, 0x1D);

		/// <summary>
		/// Returns the offsets of the pointer tables for the specified level
		/// </summary>
		/// <param name="level">The level to obtain offsets for</param>
		/// <returns>the offsets of the pointer tables for the specified level</returns>
		public static LevelPointers GetPointers(Levels level) {
			switch(level) {
				case Levels.Brinstar:
					return Brinstar;
				case Levels.Kraid:
					return Kraid;
				case Levels.Norfair:
					return Norfair;
				case Levels.Ridley:
					return Ridley;
				case Levels.Tourain:
					return Tourain;
			}
			throw new ArgumentException("Invalid level");
		}
	}

}
