using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Drawing;
using System.ComponentModel;
using System.Drawing.Imaging;

namespace Editroid
{
	class TileSelector: PictureBox
	{

		const int TileSize = 8;

		PictureBox Selection;
		byte[] _Data;

		PatternTable BrinstarTiles = new PatternTable(false);

		public TileSelector() {
			Selection = new PictureBox();
			Selection.Size = new Size(8, 8);
			Selection.Image = Properties.Resources.Select;
			Selection.BackColor = Color.Transparent;

			this.Controls.Add(Selection);
		}

		private void LoadTiles() {
			BrinstarTiles.BeginWrite();
			BrinstarTiles.LoadTiles(_Data, PatternTableRange.Brinstar);
			BrinstarTiles.EndWrite();

			
		}

        public void SetSeletion(byte index) {
            Selection.Location = new Point(
                (index % 16) * 8,
                (index / 16) * 8);

            OnTileSelected(index);
        }

		public void SetPalette(NesPalette pal, int table) {
			ColorPalette p = Image.Palette;
			pal.ApplyTable(p.Entries, 0, table * 4, 4);
			Image.Palette = p;
			Invalidate();
		}

		protected override void OnMouseDown(MouseEventArgs e) {
			base.OnMouseDown(e);

			int TileX = e.X / TileSize;
			int TileY = e.Y / TileSize;

			Selection.Left = TileX * TileSize;
			Selection.Top = TileY * TileSize;

            int Tile = TileX + TileY * 16;
            OnTileSelected(Tile);
		}

        private void OnTileSelected(int Tile) {
            if(TileSelected != null) {
                TileSelected(this, Tile);
            }
        }

		public delegate void IndexEvent(object sender, int index);
		public event IndexEvent TileSelected;

		public void SetData(Byte[] Data) {
			_Data = Data;
			LoadTiles();
		}

		private bool ShouldSerializeImage() { return false; }
		[Browsable(false), EditorBrowsable(EditorBrowsableState.Never)]
		new Image Image { get { return base.Image; } set { } }
	}

}
