using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using Editroid.Graphic;

namespace Editroid
{
	internal partial class frmComboEditor:Form
	{

		PatternTable Brinstar_DisplayTiles, Norfair_DisplayTiles, Kraid_DisplayTiles, Ridley_DisplayTiles, Tourain_DisplayTiles;
		NesPalette palBrinstar, palNorfair, palKraid, palRidley, palTourain;
		NesPalette palBrinstarAlt, palNorfairAlt, palKraidAlt, palRidleyAlt, palTourainAlt;
		PatternTable Brinstar_BlitTiles, Norfair_BlitTiles, Kraid_BlitTiles, Ridley_BlitTiles, Tourain_BlitTiles;

		NesPalette curPalette;
		NesPalette curPaletteAlt;
		Levels curLevel;
		ToolStripButton curButton;

		Rom _Rom;
		public frmComboEditor(Rom GameRom) {
			InitializeComponent();
			//_Data 
			_Rom = GameRom;

			curLevel = Levels.Brinstar;
			tsTiles.Image = BrinstarTiles;
			curButton = btnBrinstar;


			curPalette = palBrinstar;
			curPaletteAlt = palBrinstarAlt;
			SetPalettes();
			comboEditor.SetPalette(palBrinstar);

			comboEditor.SetPatterns(Brinstar_BlitTiles);
			comboEditor.SetCombos(_Rom.Brinstar.Combos);
			comboEditor.Draw();
		}

		private Bitmap BrinstarTiles {
			get {
				if(Brinstar_DisplayTiles == null) {
					Brinstar_DisplayTiles = new PatternTable(false, _Rom.data, PatternTableRange.Brinstar);
					Brinstar_BlitTiles = new PatternTable(true, _Rom.data, PatternTableRange.Brinstar);
					palBrinstar = new NesPalette(_Rom.data, NesPalette.BrinstarBackgroundOffset);
					palBrinstarAlt = new NesPalette(_Rom.data, NesPalette.BrinstarAltBackgroundOffset);

				}
				return Brinstar_DisplayTiles.Patterns;
			}
		}

		private Bitmap NorfairTiles {
			get {
				if(Norfair_DisplayTiles == null) {
					Norfair_DisplayTiles = new PatternTable(false, _Rom.data, PatternTableRange.Norfair);
					Norfair_BlitTiles = new PatternTable(true, _Rom.data, PatternTableRange.Norfair);
					palNorfair = new NesPalette(_Rom.data, NesPalette.NorfairBackgroundOffset);
					palNorfairAlt = new NesPalette(_Rom.data, NesPalette.NorfairAltBackgroundOffset);
				}
				return Norfair_DisplayTiles.Patterns;
			}
		}
		private Bitmap TourainTiles {
			get {
				if(Tourain_DisplayTiles == null) {
					Tourain_DisplayTiles = new PatternTable(false, _Rom.data, PatternTableRange.Tourain);
					Tourain_BlitTiles = new PatternTable(true, _Rom.data, PatternTableRange.Tourain);
					palTourain = new NesPalette(_Rom.data, NesPalette.TourainBackgroundOffset);
					palTourainAlt = new NesPalette(_Rom.data, NesPalette.TourainBackgroundOffset);
				}
				return Tourain_DisplayTiles.Patterns;
			}
		}
		private Bitmap KraidTiles {
			get {
				if(Kraid_DisplayTiles == null) {
					Kraid_DisplayTiles = new PatternTable(false, _Rom.data, PatternTableRange.Kraid);
					Kraid_BlitTiles = new PatternTable(true, _Rom.data, PatternTableRange.Kraid);
					palKraid = new NesPalette(_Rom.data, NesPalette.KraidBackgroundOffset);
					palKraidAlt = new NesPalette(_Rom.data, NesPalette.KraidBackgroundOffset);
				}
				return Kraid_DisplayTiles.Patterns;
			}
		}
		private Bitmap RidleyTiles {
			get {
				if(Ridley_DisplayTiles == null) {
					Ridley_DisplayTiles = new PatternTable(false, _Rom.data, PatternTableRange.Ridley);
					Ridley_BlitTiles = new PatternTable(true, _Rom.data, PatternTableRange.Ridley);
					palRidley = new NesPalette(_Rom.data, NesPalette.RidleyBackgroundOffset);
					palRidleyAlt = new NesPalette(_Rom.data, NesPalette.RidleyBackgroundOffset);
				}
				return Ridley_DisplayTiles.Patterns;
			}
		}


        private void SetPalettes() {
            SetPalettes(comboEditor.PaletteTable);
        }
        private void SetPalettes(int value) {
			if(chkAltPal.Checked) {
                tsTiles.SetPalette(curPaletteAlt, value);
				comboEditor.SetPalette(curPaletteAlt);
			} else {
                tsTiles.SetPalette(curPalette, value);
				comboEditor.SetPalette(curPalette);
			}

            comboEditor.PaletteTable = value;
		}

		private void scrComboOffset_Scroll(object sender, ScrollEventArgs e) {
			comboEditor.ScrollPosition = e.NewValue;
			comboEditor.Draw();
		}

		private void chkAltPal_CheckedChanged(object sender, EventArgs e) {
			SetPalettes();
			tsTiles.Invalidate();
			comboEditor.Draw();
		}

		private void tsTiles_TileSelected(object sender, int index) {
			comboEditor.CurrentTile = (byte)index;
            lblTile.Text = index.ToString("X").PadLeft(2, '0') + " " + ScreenDisplay.GetPhysics((byte)index).ToString();
		}

		private void btnBrinstar_Click(object sender, EventArgs e) {
            LoadLevel(Levels.Brinstar, btnBrinstar);
		}

		private void btnNorfair_Click(object sender, EventArgs e) {
            LoadLevel(Levels.Norfair, btnNorfair);
		}

		private void btnTourain_Click(object sender, EventArgs e) {
            LoadLevel(Levels.Tourain, btnTourain);
        }
		private void btnKraid_Click(object sender, EventArgs e) {
            LoadLevel(Levels.Kraid, btnKraid);
		}

        void LoadLevel(Levels levelToLoad, ToolStripButton buttonClicked) {
            if(curLevel == levelToLoad) return;

            curButton.Checked = false;
            curButton = buttonClicked;
            buttonClicked.Checked = true;

            curLevel = levelToLoad;
            tsTiles.Image = LoadTiles(levelToLoad);
            LoadPalettes(levelToLoad);
            SetPalettes();

            comboEditor.SetPalette(curPalette);
            comboEditor.SetPatterns(GetPatterns(levelToLoad));
            comboEditor.SetCombos(_Rom.GetLevel(levelToLoad).Combos);
            comboEditor.Draw();
        }

        /// <summary>Returns tiles for the specified level AND loads all tiles and palettes for that level if they arent already loaded.</summary>
        private Image LoadTiles(Levels levelToLoad) {
            switch(levelToLoad) {
                case Levels.Brinstar:
                    return BrinstarTiles;
                case Levels.Norfair:
                    return NorfairTiles;
                case Levels.Tourain:
                    return TourainTiles;
                case Levels.Kraid:
                    return KraidTiles;
                case Levels.Ridley:
                    return RidleyTiles;
            }

            return null;
        }

        private PatternTable GetPatterns(Levels levelToLoad) {
            switch(levelToLoad) {
                case Levels.Brinstar:
                    return Brinstar_BlitTiles;
                case Levels.Norfair:
                    return Norfair_BlitTiles;
                case Levels.Tourain:
                    return Tourain_BlitTiles;
                case Levels.Kraid:
                    return Kraid_BlitTiles;
                case Levels.Ridley:
                    return Ridley_BlitTiles;
            }

            return null;
        }

        private void LoadPalettes(Levels levelToLoad) {
            switch(levelToLoad) {
                case Levels.Brinstar:
                    curPalette = palBrinstar;
                    curPaletteAlt = palBrinstarAlt;
                    break;
                case Levels.Norfair:
                    curPalette = palNorfair;
                    curPaletteAlt = palNorfairAlt;
                    break;
                case Levels.Tourain:
                    curPalette = palTourain;
                    curPaletteAlt = palTourainAlt;
                    break;
                case Levels.Kraid:
                    curPalette = palKraid;
                    curPaletteAlt = palKraidAlt;
                    break;
                case Levels.Ridley:
                    curPalette = palRidley;
                    curPaletteAlt = palRidleyAlt;
                    break;
            }
        }


		private void btnRidley_Click(object sender, EventArgs e) {
			if(curLevel == Levels.Ridley) return;
			curButton.Checked = false;
			curButton = btnRidley;
			btnRidley.Checked = true;

			curLevel = Levels.Ridley;
			tsTiles.Image = RidleyTiles;

			curPalette = palRidley;
			curPaletteAlt = palRidleyAlt;
			SetPalettes();

			comboEditor.SetPalette(palRidley);
			comboEditor.SetPatterns(Ridley_BlitTiles);
			comboEditor.SetCombos(_Rom.Ridley.Combos);
			comboEditor.Draw();
		}

        private void comboEditor_UserGrabbedCombo(object sender, EventArgs e) {
            tsTiles.SetSeletion(comboEditor.CurrentTile);
        }

        private void PaletteSelection_CheckChanged(object sender, EventArgs e) {
            RadioButton Sender = sender as RadioButton;
            if(Sender == null) return;

            if(Sender.Checked) {
                SetPalettes(Int32.Parse(Sender.Text));
                comboEditor.Draw();
            }
        }
	}

    /// <summary>
    /// Represents the physics that are associated with a graphic tile.
    /// </summary>
    public enum Physics:byte
    {
        /// <summary>An impassable tile.</summary>
        Solid,
        /// <summary>An intangible tile.</summary>
        Air,
        /// <summary>A tile that can be destroyed.</summary>
        Breakable,
        /// <summary>A tile that acts as a door marker.</summary>
        Door,
        /// <summary>A tile that acts as a door marker from one adjacent horizontal room to another.</summary>
        DoorHorizontal
    }

}