using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Editroid
{
	/// <summary>
	/// Allows editing of level and sprite palettes in a Metroid ROM.
	/// </summary>
	internal partial class frmPalette:Form
	{
		/// <summary>
		/// Instantiates this component.
		/// </summary>
		public frmPalette() {
			InitializeComponent();
			bgPal.Offset = NesPalette.BrinstarBackgroundOffset;
			bgPal2.Offset = NesPalette.BrinstarAltBackgroundOffset;
			spritePal.Offset = NesPalette.BrinstarSpriteOffset;

			selectedButton = btnBrinstar;
		}

		byte[] _Data;
		/// <summary>
		/// Gets/sets the ROM image data to edit.
		/// </summary>
		public byte[] Data {
			get { return _Data; }
			set {
				_Data = value;
				bgPal.Data = value;
				bgPal2.Data = value;
				spritePal.Data = value;
			}
		}

		List<PaletteSelectionInfo> selectedItems = new List<PaletteSelectionInfo>();

		private void OnPaletteEntrySelected(object sender, PaletteSelectionInfo e) {
			if(e.AddToSelection) { // If user was holding shift...
				// Add item to selection
				selectedItems.Add(e);
			} else { // otherwise
				// Deselect all
                DeselectAll();
				selectedItems.Clear();

				// and select only clicked entry
				selectedItems.Add(e);
			}
		}

        private void DeselectAll() {
            foreach(PaletteSelectionInfo i in selectedItems) {
                i.Deselect();
            }

            selectedItems.Clear();
        }

        private void DefaultSelection() {
            DeselectAll();
            bgPal.MakeDefaultSelection();
        }

		ToolStripButton selectedButton;

		private void btnBrinstar_Click(object sender, EventArgs e) {
			selectedButton.Checked = false;
			selectedButton = (ToolStripButton)sender;
			selectedButton.Checked = true;

			bgPal.Offset = NesPalette.BrinstarBackgroundOffset;
			bgPal2.Offset = NesPalette.BrinstarAltBackgroundOffset;
			spritePal.Offset = NesPalette.BrinstarSpriteOffset;
            lblBg2.Visible = bgPal2.Visible = true;
            DefaultSelection();
		}

		private void btnNorfair_Click(object sender, EventArgs e) {
			selectedButton.Checked = false;
			selectedButton = (ToolStripButton)sender;
			selectedButton.Checked = true;

			bgPal.Offset = NesPalette.NorfairBackgroundOffset;
			bgPal2.Offset = NesPalette.NorfairAltBackgroundOffset;
			spritePal.Offset = NesPalette.NorfairSpriteOffset;
            lblBg2.Visible = bgPal2.Visible = true;
            DefaultSelection();
        }

		private void btnRidley_Click(object sender, EventArgs e) {
            lblBg2.Visible = bgPal2.Visible = false;
			selectedButton.Checked = false;
			selectedButton = (ToolStripButton)sender;
			selectedButton.Checked = true;

			bgPal.Offset = NesPalette.RidleyBackgroundOffset;
			bgPal2.Offset = NesPalette.RidleyAltBackgroundOffset;
			spritePal.Offset = NesPalette.RidleySpriteOffset;
            DefaultSelection();
        }

		private void btnKraid_Click(object sender, EventArgs e) {
            lblBg2.Visible = bgPal2.Visible = false;
			selectedButton.Checked = false;
			selectedButton = (ToolStripButton)sender;
			selectedButton.Checked = true;

			bgPal.Offset = NesPalette.KraidBackgroundOffset;
			bgPal2.Offset = NesPalette.KraidAltBackgroundOffset;
			spritePal.Offset = NesPalette.KraidSpriteOffset;
            DefaultSelection();
        }

		private void btnTourain_Click(object sender, EventArgs e) {
            lblBg2.Visible = bgPal2.Visible = false;
			selectedButton.Checked = false;
			selectedButton = (ToolStripButton)sender;
			selectedButton.Checked = true;

			bgPal.Offset = NesPalette.TourainBackgroundOffset;
			bgPal2.Offset = NesPalette.TourainAltBackgroundOffset;
			spritePal.Offset = NesPalette.TourainSpriteOffset;
            DefaultSelection();
        }

		private void picColors_MouseDown(object sender, MouseEventArgs e) {
			int PalColor = e.X / 16 + (e.Y / 16) * 16;
			// Limit palcolor to range 0 to 255
			PalColor = PalColor < 0 ? 0 : PalColor > 255 ? 255 : PalColor;

			foreach(PaletteSelectionInfo p in selectedItems) {
				p.EntryValue = PalColor;
			}
		}

	}
}