using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Editroid
{
    internal partial class frmPointers:Form
    {
        public frmPointers() {
            InitializeComponent();

        }

        Rom rom;
        public void SetRom(Rom rom) {
            this.rom = rom;
            LoadTree();

            PointerTree.ExpandAll();
        }

        private void LoadTree() {
            TreeNode root = new TreeNode("Rom Image");
            root.ImageKey = root.SelectedImageKey = "ROM";

            AddLevel(root, rom.Brinstar, "Brinstar");
            AddLevel(root, rom.Norfair, "Norfair");
            AddLevel(root, rom.Ridley, "Ridley");
            AddLevel(root, rom.Kraid, "Kraid");
            AddLevel(root, rom.Tourain, "Tourain");

            PointerTree.Nodes.Add(root);
            root.Expand();
        }

        private void AddLevel(TreeNode node, Level level, string name) {
            TreeNode levelNode = new TreeNode(name);
            levelNode.ImageKey = levelNode.SelectedImageKey = "Level";

            TreeNode combos = new TreeNode("Combos");
            combos.ImageKey = combos.SelectedImageKey = "Combo";
            combos.Tag = level.Combos.Offset;
            levelNode.Nodes.Add(combos);

            TreeNode structs = new TreeNode("Structures");
            structs.ImageKey = structs.SelectedImageKey = "Structure";
            structs.Tag = level;
            levelNode.Nodes.Add(structs);

            TreeNode pal = new TreeNode("Palettes");
            pal.ImageKey = pal.SelectedImageKey = "Palette";
            pal.Tag = level;
            levelNode.Nodes.Add(pal);

            TreeNode screen = new TreeNode("Screens");
            screen.ImageKey = screen.SelectedImageKey = "Screen";
            screen.Tag = level;
            levelNode.Nodes.Add(screen);

            node.Nodes.Add(levelNode);
        }

        private void PointerTree_AfterSelect(object sender, TreeViewEventArgs e) {
            DisplayNode(e.Node);
        }

        private void DisplayNode(TreeNode node) {
            PointerList.BeginUpdate();
            PointerList.Items.Clear();

            if(node.Tag is Int32) {
                ItemHeader.Text = "";
                ListViewItem newItem = new ListViewItem("");
                newItem.SubItems.Add(((int)(node.Tag)).ToString("X"));
                PointerList.Items.Add(newItem);
            } else if(node.Tag is Level) {
                Level level = node.Tag as Level;
                switch(node.ImageKey) {
                    case "Structure":
                        DisplayStructures(level);
                        break;
                    case "Palette":
                        DisplayPalette(level);
                        break;
                    case "Screen":
                        DisplayScreens(level);
                        break;
                }
            }

            PointerList.EndUpdate();
        }

        private void DisplayPalette(Level level) {
            ItemHeader.Text = "Name";

            // Bg palette
            ListViewItem bgItem = new ListViewItem("Background");
            bgItem.ImageKey = "Palette";
            bgItem.SubItems.Add(level.BgPalette.Offset.ToString("x"));
            PointerList.Items.Add(bgItem);

            // Sprite pal
            ListViewItem spriteItem = new ListViewItem("Sprite");
            spriteItem.ImageKey = "Palette";
            spriteItem.SubItems.Add(level.SpritePalette.Offset.ToString("x"));
            PointerList.Items.Add(spriteItem);

            // Alt palette
            if(level.LevelIdentifier == Levels.Brinstar || level.LevelIdentifier == Levels.Norfair) {
                ListViewItem bgItemAlt = new ListViewItem("Background Alt");
                bgItemAlt.ImageKey = "Palette";
                bgItemAlt.SubItems.Add(level.BgAltPalette.Offset.ToString("x"));
                PointerList.Items.Add(bgItemAlt);
            }
        }

        private void DisplayStructures(Level level) {
            ItemHeader.Text = "Index";

            int structCount = level.StructCount;
            for(int i = 0; i < structCount; i++) {
                ListViewItem newItem = new ListViewItem(i.ToString("X"));
                newItem.SubItems.Add(level.GetStruct(i).Offset.ToString("X"));

                PointerList.Items.Add(newItem);
            }
        }

        private void DisplayScreens(Level level) {
            ItemHeader.Text = "Index";

            int screenCount = level.ScreenCount;
            for(int i = 0; i < screenCount; i++) {
                ListViewItem newItem = new ListViewItem(i.ToString("X"));
                newItem.SubItems.Add(level.GetScreen(i).Offset.ToString("X"));

                PointerList.Items.Add(newItem);
            }
        }

    }
}