using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using Editroid.Graphic;

namespace Editroid
{
    internal partial class frmStruct:Form
    {

        byte[] ComboPalettes = { 1, 1, 2, 0, 2 };
        Level[] levels = new Level[5];
        PatternTable[] displayTiles = new PatternTable[5];
        PatternTable[] renderingTiles = new PatternTable[5];
        Bitmap[] combos = new Bitmap[5];

        public frmStruct() {
            InitializeComponent();
            gCurrentStruct = Graphics.FromImage(bmpCurrentStruct);
            picStruct.Image = bmpCurrentStruct;
            GlobalEventManager.Manager.ObjectSelected += new EventHandler<ScreenObjectEventArgs>(Global_ObjectSelected);
            GlobalEventManager.Manager.ObjectEdited += new EventHandler<ScreenObjectEventArgs>(Global_ObjectSelected);
        }

        private Rom rom;
        public Rom Rom {
            get { return rom; }
            set { 
                rom = value;
                LoadRomData();
            }
        }

        /// <summary>Loads data from a ROM necessary for editing features..</summary>
        private void LoadRomData() {
            levels[0] = rom.Brinstar;
            levels[1] = rom.Norfair;
            levels[2] = rom.Tourain;
            levels[3] = rom.Kraid;
            levels[4] = rom.Ridley;

            for(int i = 0; i < levels.Length; i++) {
                displayTiles[i] = new PatternTable(false, rom.data, PatternTableRange.GetBackgroundTableRanges(levels[i].LevelIdentifier));
                renderingTiles[i] = new PatternTable(true, rom.data, PatternTableRange.GetBackgroundTableRanges(levels[i].LevelIdentifier));
                //combos[i] = new Bitmap(256, 256, PixelFormat.Format8bppIndexed);

            }
            LoadLevel(Levels.Brinstar);
        }


        void Global_ObjectSelected(object sender, ScreenObjectEventArgs e) {
            if(e.Item.SelectionType == ScreenItem.ObjectType.Object) {
                // Load level and palette data
                LoadLevel(e.Level);
                selectedPal = e.Item.ThisObject.PalData;
                UpdatePalette();

                // Load tiles and combos
                tsTiles.SetPalette(levels[levelIndex].BgPalette, e.Item.ThisObject.PalData);
                tsTiles.Invalidate();
                comboEditor.PaletteTable = e.Item.ThisObject.PalData;
                comboEditor.Draw();

                // Load and display structure
                SetIndex(e.Item.Index);
            }
        }

        public void LoadLevel(Levels level) {
            // Don't reload already loaded level
            if(levelIndex == (int)level || level == Levels.Invalid)
                return;

            levelIndex = (int)level;

            // Set combo selector Palette
            //ColorPalette palette = combos[levelIndex].Palette;
            //levels[levelIndex].BgPalette.ApplyTable(palette.Entries);
            //combos[levelIndex].Palette = palette;

            if(StructIndex < 0 || StructIndex >= levels[levelIndex].StructCount) StructIndex = 0;

            tsTiles.Image = displayTiles[levelIndex].Patterns;
            
            comboEditor.SetPalette(levels[levelIndex].BgPalette);
            comboEditor.SetPatterns(renderingTiles[levelIndex]);
            comboEditor.SetCombos(levels[levelIndex].Combos);
        }

        Bitmap bmpCurrentStruct = new Bitmap(512, 512);
        Graphics gCurrentStruct;
        /// <summary>Opens the struct specified by the StructIndex property for editing.</summary>
        private void DisplayCurrentStruct() {
            if(currentStruct == null) return;

            byte[][] comboData = currentStruct.Combos;
            Rectangle source = new Rectangle(0, 0, 16, 16);
            Rectangle dest = source;
            
            gCurrentStruct.Clear(Color.Transparent);
            
            for(int y = 0; y < comboData.Length; y++) {
                for(int x = 0; x < comboData[y].Length; x++) {
                    if(comboData[y][x] != Struct.EmptyTile) {
                        source.X = 16 * (comboData[y][x] % 16);
                        source.Y = 16 * (comboData[y][x] / 16);
                        dest.X = x * 16;
                        dest.Y = y * 16;

                        //gCurrentStruct.DrawImage(combos[levelIndex], dest, source, GraphicsUnit.Pixel);
                        gCurrentStruct.DrawImage(comboEditor.BackgroundImage, dest, source, GraphicsUnit.Pixel);
                    }
                }
            }
            picStruct.Invalidate();

        }

        int levelIndex = -1;


        private Struct currentStruct;
        private int _StructIndex;
        /// <summary>
        /// Gets/sets the currently displayed combo's index
        /// </summary>
        public int StructIndex {
            get { return _StructIndex; }
            set {
                _StructIndex = value;
                currentStruct = levels[levelIndex].GetStruct(_StructIndex);
                DisplayCurrentStruct();
            }
        }

        /// <summary>
        /// Records all structs back to the ROM data.
        /// </summary>
        public void WriteStructsToRom() {
            foreach(Level l in levels) {
                for(int i = 0; i < l.StructCount; i++) {
                    l.GetStruct(i).WriteToRom();
                    l.GetStruct(i).Refresh(); // Updates struct object's cached properties
                }
            }
        }

        protected override void OnClosing(CancelEventArgs e) {
            base.OnClosing(e);

            GlobalEventManager.Manager.ObjectSelected -= new EventHandler<ScreenObjectEventArgs>(Global_ObjectSelected);
            WriteStructsToRom();
        }

        private void picStruct_MouseDown(object sender, MouseEventArgs e) {
            if(currentStruct == null) return;

            int tilex = e.X / 16;
            int tiley = e.Y / 16;
            if(e.Button == MouseButtons.Left && currentStruct.Combos.Length > tiley && 
                currentStruct.Combos[tiley].Length > tilex && currentStruct.Combos[tiley][tilex] != Struct.EmptyTile) {

                currentStruct.Combos[tiley][tilex] = (byte)(comboEditor.SelectedCombo);
                DisplayCurrentStruct();
                GlobalEventManager.Manager.OnStructureChanged((Levels)levelIndex);
            }
        }

        private void cmnStructContext_Opening(object sender, CancelEventArgs e) {
            //If a tile was clicked (not including blanks)
            if(currentStruct.Combos.Length > lastMouseY && currentStruct.Combos[lastMouseY].Length > lastMouseX && currentStruct.Combos[lastMouseY][lastMouseX] != Struct.EmptyTile) {
                clickedMouseX = lastMouseX;
                clickedMouseY = lastMouseY;

                // This data is used to determine which menu items will be available
                byte[] row = currentStruct.Combos[lastMouseY];
                int rowLength = GetRowLength(row, false);
                int rowSpacing = row.Length - rowLength;

                mnuShiftLeft.Enabled = row[0] == Struct.EmptyTile;
                mnuShiftRight.Enabled = 
                    rowSpacing < 15;
                
                // indicates that the clicked row has enough tiles that one may be swapped with an adjacent row
                bool swappable = rowLength > 1;
                mnuSwapUp.Enabled = (lastMouseY > 0 && swappable);
                mnuSwapDown.Enabled = (lastMouseY < currentStruct.Combos.Length - 1  && swappable);

                mnuSplit.Enabled = (
                    clickedMouseX > rowSpacing &&
                    clickedMouseX < row.Length - 1);
                mnuJoin.Enabled = (
                    lastMouseY < currentStruct.Combos.Length - 1 &&
                    rowLength + GetRowLength(currentStruct.Combos[lastMouseY + 1], false) + 1 < 16);

                int comboIndex = row[clickedMouseX];
                lblComboData.Text = comboIndex.ToString();
            } else {
                e.Cancel = true;
            }
        }


        /// <summary>Tracks mouse position for context menu display position.</summary>
        int lastMouseX;
        /// <summary>Tracks mouse position for context menu display position.</summary>
        int lastMouseY;
        /// <summary>Tracks mouse position for context menu display position.</summary>
        int clickedMouseX;
        /// <summary>Tracks mouse position for context menu display position.</summary>
        int clickedMouseY; 

        private void picStruct_MouseMove(object sender, MouseEventArgs e) {
            lastMouseX = e.X / 16;
            lastMouseY = e.Y / 16;
        }

        #region Row Shifting/Splitting/Joinging and relevant UI events
        private void mnuShiftLeft_Click(object sender, EventArgs e) {
            ShiftLeft(clickedMouseY);
            DisplayCurrentStruct();
        }

        /// <summary>Shifts a row left. If there are tiles on the left edge they will be cropped.</summary>
        private void ShiftLeft(int row) {
            byte[] newData = new byte[currentStruct.Combos[row].Length - 1];
            Array.Copy(currentStruct.Combos[row], 1, newData, 0, newData.Length);
            
            currentStruct.Combos[row] = newData;
        }

        /// <summary>Shifts a row right.</summary>
        private void ShiftRight(int row) {
            byte[] newData = new byte[currentStruct.Combos[row].Length + 1];
            Array.Copy(currentStruct.Combos[row], 0, newData, 1, newData.Length - 1);
            newData[0] = Struct.EmptyTile;

            currentStruct.Combos[row] = newData;
        }

        private void mnuShiftRight_Click(object sender, EventArgs e) {
            ShiftRight(clickedMouseY);
            DisplayCurrentStruct();
        }

        private void mnuSwapUp_Click(object sender, EventArgs e) {
            SwapTileUp(clickedMouseY, clickedMouseX);
            DisplayCurrentStruct();
        }

        private void SwapTileUp(int y, int x) {
            // Create new arrays for modified rows
            byte[] newData = new byte[currentStruct.Combos[y].Length - 1];
            byte[] newDataPrev = new byte[currentStruct.Combos[y - 1].Length + 1];

            // Copy old data
            Array.Copy(currentStruct.Combos[y], 0, newData, 0, x); // Copy from start to clicked tile
            Array.Copy(currentStruct.Combos[y], x + 1, newData, x, newData.Length - x); // Copy from clicked tile on
            Array.Copy(currentStruct.Combos[y - 1], 0, newDataPrev, 0, newDataPrev.Length - 1);

            // Copy last tile from bottom row to top row
            newDataPrev[newDataPrev.Length - 1] = currentStruct.Combos[y][x];

            // Write data back
            currentStruct.Combos[y] = newData;
            currentStruct.Combos[y - 1] = newDataPrev;

        }
        private void SwapTileDown(int y, int x) {
            // Create new arrays for modified rows
            byte[] newData = new byte[currentStruct.Combos[y].Length - 1];
            byte[] newDataNext = new byte[currentStruct.Combos[y + 1].Length + 1];

            // Copy old data
            Array.Copy(currentStruct.Combos[y], 0, newData, 0, x); // Copy from start to clicked tile
            Array.Copy(currentStruct.Combos[y], x + 1, newData, x, newData.Length - x); // Copy from clicked tile on
            Array.Copy(currentStruct.Combos[y + 1], 0, newDataNext, 0, newDataNext.Length - 1);

            // Copy last tile from top row to bottom row
            newDataNext[newDataNext.Length - 1] = currentStruct.Combos[y][x];
            
            // Write data back
            currentStruct.Combos[y] = newData;
            currentStruct.Combos[y + 1] = newDataNext;
        }

        private void mnuSwapDown_Click(object sender, EventArgs e) {
            SwapTileDown(clickedMouseY, clickedMouseX);
            DisplayCurrentStruct();
        }

        private void mnuSplit_Click(object sender, EventArgs e) {
            SplitRow(clickedMouseY, clickedMouseX);
            DisplayCurrentStruct();
        }

        private void mnuJoin_Click(object sender, EventArgs e) {
            JoinRows(clickedMouseY);
            DisplayCurrentStruct();
        }
        /// <summary>
        /// Joins two rows of a struct.
        /// </summary>
        /// <param name="firstRowIndex">The index of the first of the two rows to join.</param>
        private void JoinRows(int firstRowIndex) {
            byte[][] data = new byte[currentStruct.Combos.Length - 1][];

            int row = 0;

            while(row < firstRowIndex) { // Copy rows above those to be joined
                data[row] = currentStruct.Combos[row];
                row++;
            }

            data[row] = CombineRows(currentStruct.Combos[row], currentStruct.Combos[row + 1], (byte)comboEditor.SelectedCombo);
            row++;

            // Copy remaining rows
            while(row < data.Length) {
                data[row] = currentStruct.Combos[row + 1];
                row++;
            }

            currentStruct.RedefineCombos(data);
        }

        /// <summary>Returns an array that is the combination of two arrays with a joining byte.</summary>
        /// <param name="row1">The first array to join.</param>
        /// <param name="row2">The second array to join.</param>
        /// <param name="joiner">The value that will be inserted between the joined arrays.</param>
        private byte[] CombineRows(byte[] row1, byte[] row2, byte joiner) {
            int length1 = GetRowLength(row1, true);
            int length2 = GetRowLength(row2, false);

            byte[] result = new byte[length1 + length2 + 1];
            // Copy first array
            Array.Copy(row1, result, length1);
            // insert joiner
            result[length1] = joiner;
            // Copy second array, not including preceeding blanks
            Array.Copy(row2, row2.Length - length2, result, length1 + 1, length2);

            return result;
        }


        /// <summary>Gets the length of a row.</summary>
        /// <param name="row">A byte array representing a row.</param>
        /// <param name="includeBlanks">If true, preceding blank tiles will be included.</param>
        private int GetRowLength(byte[] row, bool includeBlanks) {
            if(includeBlanks) return row.Length;

            int blanks = 0;
            while(blanks < row.Length && row[blanks] == Struct.EmptyTile)
                blanks++;

            return row.Length - blanks;
        }

        private void SplitRow(int rowIndex, int splitTile) {
            byte[][] data = new byte[currentStruct.Combos.Length + 1][];

            int currentRow = 0;

            while(currentRow < rowIndex) { // Copy rows above those to be joined
                data[currentRow] = currentStruct.Combos[currentRow];
                currentRow++;
            }

            int originalLength = currentStruct.Combos[currentRow].Length;
            // Create array for first half of row
            data[currentRow] = new byte[splitTile];
            // Copy first half of row.
            Array.Copy(currentStruct.Combos[currentRow], 0, data[currentRow], 0, splitTile);
            // Create array for second half of row
            data[currentRow + 1] = new byte[originalLength - splitTile - 1];
            // Copy second half of row
            Array.Copy(currentStruct.Combos[currentRow], splitTile + 1, data[currentRow + 1], 0, originalLength - splitTile - 1);

            currentRow++;

            // Copy remaining rows.
            while(currentRow < currentStruct.Combos.Length) {
                data[currentRow + 1] = currentStruct.Combos[currentRow];
                currentRow++;
            }

            currentStruct.RedefineCombos(data);
        }

        #endregion


        int selectedPal = 0;


        private bool useAlternatePalette;

        public bool UseAlternatePalette {
            get { return useAlternatePalette; }
            set {
                useAlternatePalette = value;
                // Update combo editor palette
                comboEditor.SetPalette(GetLevelPalette());
                comboEditor.Draw();
                // Update tile editor palette
                tsTiles.SetPalette(GetLevelPalette(), comboEditor.PaletteTable);
                tsTiles.Invalidate();

                // Not sure
                UpdatePalette();

                //Todo: ensure this doesn't cause a problem if no level is loaded
                DisplayCurrentStruct();
            }
        }

        private void UpdatePalette() {
            //NesPalette levelPalette = GetLevelPalette();

            //ColorPalette imagePalette = combos[levelIndex].Palette;
            //levelPalette.ApplyTable(imagePalette.Entries);
            //combos[levelIndex].Palette = imagePalette;
        }

        /// <summary>Gets a level palette, taking into consideration the UseAlternatePalette property.</summary>
        private NesPalette GetLevelPalette() {
            Levels level = (Levels)levelIndex;
            if(useAlternatePalette && (level == Levels.Brinstar || level == Levels.Norfair))
                return levels[levelIndex].BgAltPalette;

            return levels[levelIndex].BgPalette;
        }

        internal void SetIndex(int p) {
            StructIndex = p;
            DisplayCurrentStruct();
        }

        private void tsTiles_TileSelected(object sender, int index) {
            comboEditor.CurrentTile = (byte)index;
            lblTile.Text = index.ToString("X").PadLeft(2, '0') + " " + ScreenDisplay.GetPhysics((byte)index).ToString();
        }

        private void comboEditor_ComboEdited(object sender, EventArgs e) {
            // Redraw struct incase it contains any combos that were edited
            DisplayCurrentStruct();
            GlobalEventManager.Manager.OnComboChanged((Levels)levelIndex, 0);
        }

        private void comboEditor_UserGrabbedCombo(object sender, EventArgs e) {
            tsTiles.SetSeletion(comboEditor.CurrentTile);
        }


    }
}